/*
 * Least Cost Routing plugin for the Video Disk Recorder
 *
 * See the README file for copyright information and how to reach the author.
 *
 * $Id$
 */

#include "lcr.h"

cString destinationData[7], priceData[7], pulseData[7];
const char * destinationNamesLocalized[NUMBER_ENTRIES];

#if VDRVERSNUM && VDRVERSNUM < 10321
#error "Sorry, VDR < 1.3.21 not tested - delete this line on your own risk to be a TESTER ;-)!"
#endif

cPluginLcr::cPluginLcr(void)
{
  // Initialize any member variables here.
  // DON'T DO ANYTHING ELSE THAT MAY HAVE SIDE EFFECTS, REQUIRE GLOBAL
  // VDR OBJECTS TO EXIST OR PRODUCE ANY OUTPUT!
  vLcrOsd = NULL;
}

cPluginLcr::~cPluginLcr()
{
  // Clean up after yourself!
}

const char *cPluginLcr::CommandLineHelp(void)
{
  // Return a string that describes all known command line options.
  return NULL;
}

bool cPluginLcr::ProcessArgs(int argc, char *argv[])
{
  // Implement command line argument processing here if applicable.
  return true;
}

bool cPluginLcr::Initialize(void)
{
  // Initialize any background activities the plugin shall perform.
  return true;
}

bool cPluginLcr::Start(void)
{
  // Start any background activities the plugin shall perform.
  RegisterI18n(Phrases);
  return true;
}

void cPluginLcr::Stop(void)
{
  // Stop the background activities.
}

void cPluginLcr::Housekeeping(void)
{
  // Perform any cleanup or other regular tasks.
}

cOsdObject *cPluginLcr::MainMenuAction(void)
{
  // Perform the action when selected from the main VDR menu.
  vLcrOsd = new cLcrOsd();
  return vLcrOsd;
}

bool cPluginLcr::SetupParse(const char *Name, const char *Value)
{
  // Parse your own setup parameters and store their values.
  if      (!strcasecmp(Name, "ZoomIn"))                 lcrConfig.zoomIn                = atoi(Value);
  else if (!strcasecmp(Name, "PreferredEntry"))         lcrConfig.preferredEntry        = atoi(Value);
#if VDRVERSNUM >= 10330
  else if (!strcasecmp(Name, "OtherDisplays"))          lcrConfig.otherDisplays         = atoi(Value);
#endif
  else if (!strcasecmp(Name, "HighlightEntry"))         lcrConfig.highlightEntry        = atoi(Value);
  else if (!strcasecmp(Name, "Destination1"))           lcrConfig.destination[1]        = atoi(Value);
  else if (!strcasecmp(Name, "Destination2"))           lcrConfig.destination[2]        = atoi(Value);
  else if (!strcasecmp(Name, "Destination3"))           lcrConfig.destination[3]        = atoi(Value);
  else if (!strcasecmp(Name, "Destination4"))           lcrConfig.destination[4]        = atoi(Value);
  else if (!strcasecmp(Name, "Destination5"))           lcrConfig.destination[5]        = atoi(Value);
  else if (!strcasecmp(Name, "Destination6"))           lcrConfig.destination[6]        = atoi(Value);
  else if (!strcasecmp(Name, "Destination7"))           lcrConfig.destination[7]        = atoi(Value);
  else if (!strcasecmp(Name, "Destination8"))           lcrConfig.destination[8]        = atoi(Value);
  else if (!strcasecmp(Name, "Destination9"))           lcrConfig.destination[9]        = atoi(Value);
  else if (!strcasecmp(Name, "Destination0"))           lcrConfig.destination[0]        = atoi(Value);
  else if (!strcasecmp(Name, "LogoFrame"))              lcrConfig.logoFrame             = atoi(Value);
  else if (!strcasecmp(Name, "HideMenu"))               lcrConfig.hidemenu              = atoi(Value);
  else if (!strcasecmp(Name, "OSDOffsetX"))             lcrConfig.osdoffsetX            = atoi(Value);
  else if (!strcasecmp(Name, "OSDOffsetY"))             lcrConfig.osdoffsetY            = atoi(Value);
  else if (!strcasecmp(Name, "Theme"))                  lcrConfig.theme                 = atoi(Value);
  else
    return false;

  return true;
}

cMenuLcrSetup::cMenuLcrSetup(void)
{
  themes[eLcrThemeClassic]       = tr("Classic");
  themes[eLcrThemeElchi]         = tr("Elchi");
  themes[eLcrThemeDeepBlue]      = tr("DeepBlue");
  themes[eLcrThemeMoronimo]      = tr("Moronimo");

  Setup();
}

void cMenuLcrSetup::Setup(void)
{
    int i;
    // do the translations for the selectable strings in the setup
//    const char * destinationNamesLocalized[NUMBER_ENTRIES];
    for ( i=0; i < NUMBER_ENTRIES; i++ )
    {
        destinationNamesLocalized[i] = tr(destinationNames[i]);
    }
  int current = Current();

  Clear();
  Add(new cMenuEditBoolItem(  tr("Zoom-in"),                     &lcrConfig.zoomIn,                 tr("no"),           tr("yes")));
  Add(new cMenuEditIntItem(   tr("Preferred entry"),             &lcrConfig.preferredEntry,         0,                  7));
#if VDRVERSNUM >= 10330
  Add(new cMenuEditBoolItem(  tr("Display on lcdproc & co."),    &lcrConfig.otherDisplays,          tr("no"),           tr("yes")));
#endif
  Add(new cMenuEditBoolItem(  tr("Highlight entry"),             &lcrConfig.highlightEntry,         tr("no"),           tr("yes")));
  Add(new cMenuEditStraItem(  tr("Destination 1"),               &lcrConfig.destination[1],         NUMBER_ENTRIES,     destinationNamesLocalized));
  Add(new cMenuEditStraItem(  tr("Destination 2"),               &lcrConfig.destination[2],         NUMBER_ENTRIES,     destinationNamesLocalized));
  Add(new cMenuEditStraItem(  tr("Destination 3"),               &lcrConfig.destination[3],         NUMBER_ENTRIES,     destinationNamesLocalized));
  Add(new cMenuEditStraItem(  tr("Destination 4"),               &lcrConfig.destination[4],         NUMBER_ENTRIES,     destinationNamesLocalized));
  Add(new cMenuEditStraItem(  tr("Destination 5"),               &lcrConfig.destination[5],         NUMBER_ENTRIES,     destinationNamesLocalized));
  Add(new cMenuEditStraItem(  tr("Destination 6"),               &lcrConfig.destination[6],         NUMBER_ENTRIES,     destinationNamesLocalized));
  Add(new cMenuEditStraItem(  tr("Destination 7"),               &lcrConfig.destination[7],         NUMBER_ENTRIES,     destinationNamesLocalized));
  Add(new cMenuEditStraItem(  tr("Destination 8"),               &lcrConfig.destination[8],         NUMBER_ENTRIES,     destinationNamesLocalized));
  Add(new cMenuEditStraItem(  tr("Destination 9"),               &lcrConfig.destination[9],         NUMBER_ENTRIES,     destinationNamesLocalized));
  Add(new cMenuEditStraItem(  tr("Destination 0"),               &lcrConfig.destination[0],         NUMBER_ENTRIES,     destinationNamesLocalized));
  Add(new cMenuEditBoolItem(  tr("Frame around Logo"),           &lcrConfig.logoFrame,              tr("no"),           tr("yes")));
  Add(new cMenuEditBoolItem(  tr("Hide main menu entry"),        &lcrConfig.hidemenu,               tr("no"),           tr("yes")));
  Add(new cMenuEditStraItem(  tr("Theme"),                       &lcrConfig.theme,                  eLcrThemeMaxNumber, themes));
  Add(new cMenuEditIntItem(   tr("Horizontal offset"),           &lcrConfig.osdoffsetX,             -50,                50));
  Add(new cMenuEditIntItem(   tr("Vertical offset"),             &lcrConfig.osdoffsetY,             0,                  300));

  SetCurrent(Get(current));
  Display();
}

void cMenuLcrSetup::Store(void)
{
  SetupStore("ZoomIn",                  lcrConfig.zoomIn);
  SetupStore("PreferredEntry",          lcrConfig.preferredEntry);
#if VDRVERSNUM >= 10330
  SetupStore("OtherDisplays",           lcrConfig.otherDisplays);
#endif
  SetupStore("HighlightEntry",          lcrConfig.highlightEntry);
  SetupStore("Destination1",            lcrConfig.destination[1]);
  SetupStore("Destination2",            lcrConfig.destination[2]);
  SetupStore("Destination3",            lcrConfig.destination[3]);
  SetupStore("Destination4",            lcrConfig.destination[4]);
  SetupStore("Destination5",            lcrConfig.destination[5]);
  SetupStore("Destination6",            lcrConfig.destination[6]);
  SetupStore("Destination7",            lcrConfig.destination[7]);
  SetupStore("Destination8",            lcrConfig.destination[8]);
  SetupStore("Destination9",            lcrConfig.destination[9]);
  SetupStore("Destination0",            lcrConfig.destination[0]);
  SetupStore("LogoFrame",               lcrConfig.logoFrame);
  SetupStore("HideMenu",                lcrConfig.hidemenu);
  SetupStore("Theme",                   lcrConfig.theme);
  SetupStore("OSDOffsetX",              lcrConfig.osdoffsetX);
  SetupStore("OSDOffsetY",              lcrConfig.osdoffsetY);
}

eOSState cMenuLcrSetup::ProcessKey(eKeys Key)
{
//  int oldAnalyzestream = lcrConfig.analyzestream;

  eOSState state = cMenuSetupPage::ProcessKey(Key);

  if (Key != kNone) {
     Setup();
  }

  return state;
}

cMenuSetupPage *cPluginLcr::SetupMenu(void)
{
  // Return a setup menu in case the plugin supports one.
  return new cMenuLcrSetup;
}

bool cPluginLcr::Service(const char *Id, void *Data)
{
    if (strcmp(Id,"LcrService-v1.0") == 0 && destinationData[0] != NULL && strcmp(destinationData[0],"---")
      && lcrConfig.otherDisplays == 1
    )
    {
        if ( Data )
        {
            LcrService_v1_0 *data = (LcrService_v1_0*)Data;
            vLcrOsd->getServiceData( data );
        }
        return true;
    }
    return false;
}

VDRPLUGINCREATOR(cPluginLcr); // Don't touch this!
